@@ -38,6 +38,7 @@
 #include <nghttp2/nghttp2.h>
 
 #include "shrpx_io_control.h"
+#include "shrpx_log_config.h"
 #include "http2.h"
 #include "memchunk.h"
 #include "allocator.h"
@@ -76,7 +77,6 @@ public:
   // Returns pointer to the header field with the name |name|.  If
   // multiple header have |name| as name, return last occurrence from
   // the beginning.  If no such header is found, returns nullptr.
-  // This function must be called after headers are indexed
   const HeaderRefs::value_type *header(int32_t token) const;
   HeaderRefs::value_type *header(int32_t token);
   // Returns pointer to the header field with the name |name|.  If no
@@ -86,6 +86,11 @@ public:
   void add_header_token(const StringRef &name, const StringRef &value,
                         bool no_index, int32_t token);
 
+  // Adds header field name |name|.  First, the copy of header field
+  // name pointed by name.c_str() of length name.size() is made, and
+  // stored.
+  void alloc_add_header_name(const StringRef &name);
+
   void append_last_header_key(const char *data, size_t len);
   void append_last_header_value(const char *data, size_t len);
 
@@ -101,6 +106,11 @@ public:
   void add_trailer_token(const StringRef &name, const StringRef &value,
                          bool no_index, int32_t token);
 
+  // Adds trailer field name |name|.  First, the copy of trailer field
+  // name pointed by name.c_str() of length name.size() is made, and
+  // stored.
+  void alloc_add_trailer_name(const StringRef &name);
+
   void append_last_trailer_key(const char *data, size_t len);
   void append_last_trailer_value(const char *data, size_t len);
 
@@ -143,6 +153,8 @@ struct Request {
   }
 
   FieldStore fs;
+  // Timestamp when all request header fields are received.
+  std::shared_ptr<Timestamp> tstamp;
   // Request scheme.  For HTTP/2, this is :scheme header field value.
   // For HTTP/1.1, this is deduced from URI or connection.
   StringRef scheme;
@@ -303,6 +315,7 @@ public:
   void set_request_pending(bool f);
   bool get_request_pending() const;
   void set_request_header_sent(bool f);
+  bool get_request_header_sent() const;
   // Returns true if request is ready to be submitted to downstream.
   // When sending pending request, get_request_pending() should be
   // checked too because this function may return true when
@@ -335,6 +348,9 @@ public:
   // True if the response is non-final (1xx status code).  Note that
   // if connection was upgraded, 101 status code is treated as final.
   bool get_non_final_response() const;
+  // True if protocol version used by client supports non final
+  // response.  Only HTTP/1.1 and HTTP/2 clients support it.
+  bool supports_non_final_response() const;
   void set_expect_final_response(bool f);
   bool get_expect_final_response() const;
 
@@ -394,6 +410,8 @@ public:
 
   const DownstreamAddr *get_addr() const;
 
+  void set_accesslog_written(bool f);
+
   enum {
     EVENT_ERROR = 0x1,
     EVENT_TIMEOUT = 0x2,
@@ -477,6 +495,8 @@ private:
   bool request_pending_;
   // true if downstream request header is considered to be sent.
   bool request_header_sent_;
+  // true if access.log has been written.
+  bool accesslog_written_;
 };
 
 } // namespace shrpx
