@@ -70,6 +70,8 @@ constexpr auto NGHTTP2_H2_14 = StringRef::from_lit("h2-14");
 constexpr auto NGHTTP2_H1_1_ALPN = StringRef::from_lit("\x8http/1.1");
 constexpr auto NGHTTP2_H1_1 = StringRef::from_lit("http/1.1");
 
+constexpr size_t NGHTTP2_MAX_UINT64_DIGITS = str_size("18446744073709551615");
+
 namespace util {
 
 inline bool is_alpha(const char c) {
@@ -82,6 +84,9 @@ inline bool is_hex_digit(const char c) {
   return is_digit(c) || ('A' <= c && c <= 'F') || ('a' <= c && c <= 'f');
 }
 
+// Returns true if |s| is hex string.
+bool is_hex_string(const StringRef &s);
+
 bool in_rfc3986_unreserved_chars(const char c);
 
 bool in_rfc3986_sub_delims(const char c);
@@ -129,15 +134,15 @@ std::string percent_decode(InputIt first, InputIt last) {
 StringRef percent_decode(BlockAllocator &balloc, const StringRef &src);
 
 // Percent encode |target| if character is not in token or '%'.
-std::string percent_encode_token(const std::string &target);
+StringRef percent_encode_token(BlockAllocator &balloc, const StringRef &target);
 
 // Returns quotedString version of |target|.  Currently, this function
 // just replace '"' with '\"'.
-std::string quote_string(const std::string &target);
+StringRef quote_string(BlockAllocator &balloc, const StringRef &target);
 
 std::string format_hex(const unsigned char *s, size_t len);
 
-template <size_t N> std::string format_hex(const unsigned char(&s)[N]) {
+template <size_t N> std::string format_hex(const unsigned char (&s)[N]) {
   return format_hex(s, N);
 }
 
@@ -145,22 +150,43 @@ template <size_t N> std::string format_hex(const std::array<uint8_t, N> &s) {
   return format_hex(s.data(), s.size());
 }
 
+StringRef format_hex(BlockAllocator &balloc, const StringRef &s);
+
+// decode_hex decodes hex string |s|, returns the decoded byte string.
+// This function assumes |s| is hex string, that is is_hex_string(s)
+// == true.
+StringRef decode_hex(BlockAllocator &balloc, const StringRef &s);
+
+// Returns given time |t| from epoch in HTTP Date format (e.g., Mon,
+// 10 Oct 2016 10:25:58 GMT).
 std::string http_date(time_t t);
+// Writes given time |t| from epoch in HTTP Date format into the
+// buffer pointed by |res|.  The buffer must be at least 29 bytes
+// long.  This function returns the one beyond the last position.
+char *http_date(char *res, time_t t);
 
 // Returns given time |t| from epoch in Common Log format (e.g.,
 // 03/Jul/2014:00:19:38 +0900)
 std::string common_log_date(time_t t);
+// Writes given time |t| from epoch in Common Log format into the
+// buffer pointed by |res|.  The buffer must be at least 26 bytes
+// long.  This function returns the one beyond the last position.
+char *common_log_date(char *res, time_t t);
 
 // Returns given millisecond |ms| from epoch in ISO 8601 format (e.g.,
-// 2014-11-15T12:58:24.741Z)
+// 2014-11-15T12:58:24.741Z or 2014-11-15T12:58:24.741+09:00)
 std::string iso8601_date(int64_t ms);
+// Writes given time |t| from epoch in ISO 8601 format into the buffer
+// pointed by |res|.  The buffer must be at least 29 bytes long.  This
+// function returns the one beyond the last position.
+char *iso8601_date(char *res, int64_t ms);
 
 time_t parse_http_date(const StringRef &s);
 
 char upcase(char c);
 
 inline char lowcase(char c) {
-  static unsigned char tbl[] = {
+  constexpr static unsigned char tbl[] = {
       0,   1,   2,   3,   4,   5,   6,   7,   8,   9,   10,  11,  12,  13,  14,
       15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
       30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,
@@ -216,7 +242,7 @@ template <typename S, typename T> bool istarts_with(const S &a, const T &b) {
 }
 
 template <typename T, typename CharT, size_t N>
-bool istarts_with_l(const T &a, const CharT(&b)[N]) {
+bool istarts_with_l(const T &a, const CharT (&b)[N]) {
   return istarts_with(a.begin(), a.end(), b, b + N - 1);
 }
 
@@ -234,7 +260,7 @@ template <typename T, typename S> bool ends_with(const T &a, const S &b) {
 }
 
 template <typename T, typename CharT, size_t N>
-bool ends_with_l(const T &a, const CharT(&b)[N]) {
+bool ends_with_l(const T &a, const CharT (&b)[N]) {
   return ends_with(a.begin(), a.end(), b, b + N - 1);
 }
 
@@ -252,7 +278,7 @@ template <typename T, typename S> bool iends_with(const T &a, const S &b) {
 }
 
 template <typename T, typename CharT, size_t N>
-bool iends_with_l(const T &a, const CharT(&b)[N]) {
+bool iends_with_l(const T &a, const CharT (&b)[N]) {
   return iends_with(a.begin(), a.end(), b, b + N - 1);
 }
 
@@ -270,12 +296,12 @@ template <typename T, typename S> bool strieq(const T &a, const S &b) {
 }
 
 template <typename CharT, typename InputIt, size_t N>
-bool strieq_l(const CharT(&a)[N], InputIt b, size_t blen) {
+bool strieq_l(const CharT (&a)[N], InputIt b, size_t blen) {
   return strieq(a, a + (N - 1), b, b + blen);
 }
 
 template <typename CharT, size_t N, typename T>
-bool strieq_l(const CharT(&a)[N], const T &b) {
+bool strieq_l(const CharT (&a)[N], const T &b) {
   return strieq(a, a + (N - 1), b.begin(), b.end());
 }
 
@@ -292,12 +318,12 @@ template <typename T, typename S> bool streq(const T &a, const S &b) {
 }
 
 template <typename CharT, typename InputIt, size_t N>
-bool streq_l(const CharT(&a)[N], InputIt b, size_t blen) {
+bool streq_l(const CharT (&a)[N], InputIt b, size_t blen) {
   return streq(a, a + (N - 1), b, b + blen);
 }
 
 template <typename CharT, size_t N, typename T>
-bool streq_l(const CharT(&a)[N], const T &b) {
+bool streq_l(const CharT (&a)[N], const T &b) {
   return streq(a, a + (N - 1), b.begin(), b.end());
 }
 
@@ -358,7 +384,7 @@ template <typename T, typename OutputIt> OutputIt utos(OutputIt dst, T n) {
 
 template <typename T>
 StringRef make_string_ref_uint(BlockAllocator &balloc, T n) {
-  auto iov = make_byte_ref(balloc, str_size("18446744073709551615") + 1);
+  auto iov = make_byte_ref(balloc, NGHTTP2_MAX_UINT64_DIGITS + 1);
   auto p = iov.base;
   p = util::utos(p, n);
   *p = '\0';
@@ -424,9 +450,10 @@ template <typename T> std::string utox(T n) {
 }
 
 void to_token68(std::string &base64str);
-void to_base64(std::string &token68str);
 
-void show_candidates(const char *unkopt, option *options);
+StringRef to_base64(BlockAllocator &balloc, const StringRef &token68str);
+
+void show_candidates(const char *unkopt, const option *options);
 
 bool has_uri_field(const http_parser_url &u, http_parser_url_fields field);
 
@@ -436,6 +463,9 @@ bool fieldeq(const char *uri1, const http_parser_url &u1, const char *uri2,
 bool fieldeq(const char *uri, const http_parser_url &u,
              http_parser_url_fields field, const char *t);
 
+bool fieldeq(const char *uri, const http_parser_url &u,
+             http_parser_url_fields field, const StringRef &t);
+
 StringRef get_uri_field(const char *uri, const http_parser_url &u,
                         http_parser_url_fields field);
 
@@ -461,23 +491,8 @@ std::string numeric_name(const struct sockaddr *sa, socklen_t salen);
 // IPv6 address, address is enclosed by square brackets ([]).
 std::string to_numeric_addr(const Address *addr);
 
-// Makes internal copy of stderr (and possibly stdout in the future),
-// which is then used as pointer to /dev/stderr or /proc/self/fd/2
-void store_original_fds();
-
-// Restores the original stderr that was stored with copy_original_fds
-// Used just before execv
-void restore_original_fds();
-
-// Closes |fd| which was returned by open_log_file (see below)
-// and sets it to -1. In the case that |fd| points to stdout or
-// stderr, or is -1, the descriptor is not closed (but still set to -1).
-void close_log_file(int &fd);
-
-// Opens |path| with O_APPEND enabled.  If file does not exist, it is
-// created first.  This function returns file descriptor referring the
-// opened file if it succeeds, or -1.
-int open_log_file(const char *path);
+// Sets |port| to |addr|.
+void set_port(Address &addr, uint16_t port);
 
 // Returns ASCII dump of |data| of length |len|.  Only ASCII printable
 // characters are preserved.  Other characters are replaced with ".".
@@ -533,29 +548,55 @@ std::vector<std::string> parse_config_str_list(const StringRef &s,
 // treated as a part of substring.
 std::vector<StringRef> split_str(const StringRef &s, char delim);
 
-// Returns given time |tp| in Common Log format (e.g.,
-// 03/Jul/2014:00:19:38 +0900)
-// Expected type of |tp| is std::chrono::timepoint
-template <typename T> std::string format_common_log(const T &tp) {
+// Writes given time |tp| in Common Log format (e.g.,
+// 03/Jul/2014:00:19:38 +0900) in buffer pointed by |out|.  The buffer
+// must be at least 27 bytes, including terminal NULL byte.  Expected
+// type of |tp| is std::chrono::time_point.  This function returns
+// StringRef wrapping the buffer pointed by |out|, and this string is
+// terminated by NULL.
+template <typename T> StringRef format_common_log(char *out, const T &tp) {
   auto t =
       std::chrono::duration_cast<std::chrono::seconds>(tp.time_since_epoch());
-  return common_log_date(t.count());
+  auto p = common_log_date(out, t.count());
+  *p = '\0';
+  return StringRef{out, p};
 }
 
 // Returns given time |tp| in ISO 8601 format (e.g.,
-// 2014-11-15T12:58:24.741Z)
-// Expected type of |tp| is std::chrono::timepoint
+// 2014-11-15T12:58:24.741Z or 2014-11-15T12:58:24.741+09:00).
+// Expected type of |tp| is std::chrono::time_point
 template <typename T> std::string format_iso8601(const T &tp) {
   auto t = std::chrono::duration_cast<std::chrono::milliseconds>(
       tp.time_since_epoch());
   return iso8601_date(t.count());
 }
 
-// Returns given time |tp| in HTTP date format.
-template <typename T> std::string format_http_date(const T &tp) {
+// Writes given time |tp| in ISO 8601 format (e.g.,
+// 2014-11-15T12:58:24.741Z or 2014-11-15T12:58:24.741+09:00) in
+// buffer pointed by |out|.  The buffer must be at least 30 bytes,
+// including terminal NULL byte.  Expected type of |tp| is
+// std::chrono::time_point.  This function returns StringRef wrapping
+// the buffer pointed by |out|, and this string is terminated by NULL.
+template <typename T> StringRef format_iso8601(char *out, const T &tp) {
+  auto t = std::chrono::duration_cast<std::chrono::milliseconds>(
+      tp.time_since_epoch());
+  auto p = iso8601_date(out, t.count());
+  *p = '\0';
+  return StringRef{out, p};
+}
+
+// Writes given time |tp| in HTTP Date format (e.g., Mon, 10 Oct 2016
+// 10:25:58 GMT) in buffer pointed by |out|.  The buffer must be at
+// least 30 bytes, including terminal NULL byte.  Expected type of
+// |tp| is std::chrono::time_point.  This function returns StringRef
+// wrapping the buffer pointed by |out|, and this string is terminated
+// by NULL.
+template <typename T> StringRef format_http_date(char *out, const T &tp) {
   auto t =
       std::chrono::duration_cast<std::chrono::seconds>(tp.time_since_epoch());
-  return http_date(t.count());
+  auto p = http_date(out, t.count());
+  *p = '\0';
+  return StringRef{out, p};
 }
 
 // Return the system precision of the template parameter |Clock| as
@@ -630,12 +671,16 @@ std::string format_duration(double t);
 // Creates "host:port" string using given |host| and |port|.  If
 // |host| is numeric IPv6 address (e.g., ::1), it is enclosed by "["
 // and "]".  If |port| is 80 or 443, port part is omitted.
-std::string make_http_hostport(const StringRef &host, uint16_t port);
+StringRef make_http_hostport(BlockAllocator &balloc, const StringRef &host,
+                             uint16_t port);
 
 // Just like make_http_hostport(), but doesn't treat 80 and 443
 // specially.
 std::string make_hostport(const StringRef &host, uint16_t port);
 
+StringRef make_hostport(BlockAllocator &balloc, const StringRef &host,
+                        uint16_t port);
+
 // Dumps |src| of length |len| in the format similar to `hexdump -C`.
 void hexdump(FILE *out, const uint8_t *src, size_t len);
 
@@ -665,20 +710,23 @@ uint64_t get_uint64(const uint8_t *data);
 int read_mime_types(std::map<std::string, std::string> &res,
                     const char *filename);
 
-template <typename Generator>
-std::string random_alpha_digit(Generator &gen, size_t len) {
-  std::string res;
-  res.reserve(len);
+// Fills random alpha and digit byte to the range [|first|, |last|).
+// Returns the one beyond the |last|.
+template <typename OutputIt, typename Generator>
+OutputIt random_alpha_digit(OutputIt first, OutputIt last, Generator &gen) {
+  // If we use uint8_t instead char, gcc 6.2.0 complains by shouting
+  // char-array initialized from wide string.
+  static constexpr char s[] =
+      "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
   std::uniform_int_distribution<> dis(0, 26 * 2 + 10 - 1);
-  for (; len > 0; --len) {
-    res += "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"[dis(
-        gen)];
+  for (; first != last; ++first) {
+    *first = s[dis(gen)];
   }
-  return res;
+  return first;
 }
 
 template <typename OutputIterator, typename CharT, size_t N>
-OutputIterator copy_lit(OutputIterator it, CharT(&s)[N]) {
+OutputIterator copy_lit(OutputIterator it, CharT (&s)[N]) {
   return std::copy_n(s, N - 1, it);
 }
 
@@ -691,6 +739,14 @@ uint32_t hash32(const StringRef &s);
 // returns 0 if it succeeds, or -1.
 int sha256(uint8_t *buf, const StringRef &s);
 
+// Returns host from |hostport|.  If host cannot be found in
+// |hostport|, returns empty string.  The returned string might not be
+// NULL-terminated.
+StringRef extract_host(const StringRef &hostport);
+
+// Returns new std::mt19937 object.
+std::mt19937 make_mt19937();
+
 } // namespace util
 
 } // namespace nghttp2
